@@ -7,7 +7,6 @@ from typing import (
     Dict,
     Iterable,
     List,
-    NamedTuple,
     Optional,
     Sequence,
     Set,
@@ -28,14 +27,17 @@ from sciencebeam_parser.document.layout_document import (
     LayoutDocument
 )
 from sciencebeam_parser.models.data import (
+    NEW_DOCUMENT_MARKER,
     AppFeaturesContext,
     DocumentFeaturesContext,
     LabeledLayoutModelData,
+    LabeledLayoutToken,
     LayoutModelData,
-    ModelDataGenerator
+    ModelDataGenerator,
+    NewDocumentMarker
 )
 from sciencebeam_parser.models.extract import ModelSemanticExtractor
-from sciencebeam_parser.models.training_data import TeiTrainingDataGenerator
+from sciencebeam_parser.models.training_data import TeiTrainingDataGenerator, TrainingTeiParser
 from sciencebeam_parser.document.semantic_document import SemanticContentWrapper
 from sciencebeam_parser.models.model_impl import ModelImpl, T_ModelImplFactory
 from sciencebeam_parser.utils.lazy import LazyLoaded, Preloadable
@@ -56,18 +58,6 @@ class LayoutModelLabel:
     layout_token: Optional[LayoutToken] = field(repr=False, default=None)
 
 
-class LabeledLayoutToken(NamedTuple):
-    label: str
-    layout_token: LayoutToken
-
-
-class NewDocumentMarker:
-    pass
-
-
-NEW_DOCUMENT_MARKER = NewDocumentMarker()
-
-
 def iter_entities_including_other(seq: List[str]) -> Iterable[Tuple[str, int, int]]:
     """
     Similar to get_entities, but also other (`O`) tag
@@ -254,6 +244,10 @@ class Model(ABC, Preloadable):
     def get_tei_training_data_generator(self) -> TeiTrainingDataGenerator:
         raise NotImplementedError()
 
+    # @abstractmethod
+    def get_training_tei_parser(self) -> TrainingTeiParser:
+        raise NotImplementedError()
+
     def _load_model_impl(self) -> ModelImpl:
         assert self._model_impl_factory, 'model impl factory required'
         LOGGER.info('creating model impl: %r', self._model_impl_factory)
